import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

enum AdLoadState { loading, loadError, loadCompleted }
typedef void OnLoadStateCallback(AdLoadState adLoadState);

class NativeAdmobController {
  final _key = UniqueKey();
  String get id => _key.toString();

  final _stateChanged = StreamController<AdLoadState>.broadcast();
  Stream<AdLoadState> get stateChanged => _stateChanged.stream;

  OnLoadStateCallback onLoadStateCallback;

  final MethodChannel _pluginChannel = const MethodChannel("admob_network_ad");
  MethodChannel _channel;
  String _adUnitID;

  NativeAdmobController() {
    _channel = MethodChannel(id);
    _channel.setMethodCallHandler(_handleMessages);
    _pluginChannel.invokeMethod("initController", {
      "controllerID": id,
    });
  }

  void dispose() {
    _pluginChannel.invokeMethod("disposeController", {
      "controllerID": id,
    });
  }

  Future<Null> _handleMessages(MethodCall call) async {
    switch (call.method) {
      case "loading":
        _stateChanged.add(AdLoadState.loading);
        loadStateCallback(AdLoadState.loading);
        break;

      case "loadError":
        _stateChanged.add(AdLoadState.loadError);
        loadStateCallback(AdLoadState.loadError);
        break;

      case "loadCompleted":
        _stateChanged.add(AdLoadState.loadCompleted);
        loadStateCallback(AdLoadState.loadCompleted);
        break;
    }
  }

  loadStateCallback(AdLoadState adLoadState) {
    if (null != onLoadStateCallback) {
      onLoadStateCallback(adLoadState);
    }
  }

  /// Change the ad unit ID
  void setAdUnitID(String adUnitID) {
    _adUnitID = adUnitID;
    _channel.invokeMethod("setAdUnitID", {
      "adUnitID": adUnitID,
    });
  }

  void reloadAd({bool forceRefresh = false}) {
    if (_adUnitID == null) return;

    _channel.invokeMethod("reloadAd", {
      "forceRefresh": forceRefresh,
    });
  }
}
